/*-
 * ~~~~~~licensing~~~~~~
 * object-producer-core
 * ==========
 * Copyright (C) 2020 - 2022 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor;

import java.util.HashMap;
import java.util.Map;

public class CommandInstructions {

	protected Map<String, Object> instructionsMap = new HashMap<>();
	
	public CommandInstructions() {
		super();
	}
	
	public CommandInstructions(Map<String, Object> instructionsMap) {
		super();
		load(instructionsMap);
	}
	
	public void load(Map<String, Object> instructionsMap) {
		this.instructionsMap = instructionsMap;
	}
	
	public boolean has(String name) {
		return this.instructionsMap.containsKey(name);
	}
	
	public Boolean getBoolean(String name) {
		if (!this.has(name))
			return null;
		Object value = this.instructionsMap.get(name);
		
		if (value instanceof Boolean)
			return (Boolean)value;
		
		if (value instanceof String)
			return "true".equals(((String)value).toLowerCase());
		
		if (value instanceof Number)
			return ((Number)value).longValue() > 0;
			
		if (value instanceof Long)
			return ((Long)value).longValue() > 0;
		
		return value!=null;
	}
	
	public Object get(String name) {
		return this.instructionsMap.get(name);
	}
	
	public String getString(String name) {
		if (!has(name))
			return null;
		Object value = instructionsMap.get(name);
		return value!=null?value.toString():"";
	}
	
	public Long getLong(String name) {
		if (!has(name))
			return null;
		Object value = instructionsMap.get(name);
		if (value instanceof Number)
			return ((Number)value).longValue();
		return (long)-1;
	}
	
}
