/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2022 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.commands;

import java.util.Map;
import java.util.stream.Collectors;

import ru.entaxy.platform.core.artifact.Artifact;
import ru.entaxy.platform.core.artifact.DeployedArtifact;
import ru.entaxy.platform.core.artifact.service.ArtifactService;
import ru.entaxy.platform.core.producer.api.EntaxyProducerService;
import ru.entaxy.platform.core.producer.api.ProducerResult;
import ru.entaxy.platform.core.producer.api.ProducerResult.CommandResult;
import ru.entaxy.platform.core.producer.executor.AbstractCommandExecutor;
import ru.entaxy.platform.core.producer.executor.CommandExecutor;
import ru.entaxy.platform.core.producer.executor.support.ArtifactList;
import ru.entaxy.platform.core.producer.executor.support.ArtifactsHelper;
import ru.entaxy.platform.core.producer.executor.support.DeployedArtifactList;

@CommandExecutor(id = "deploy", predecessors = {"build"})
public class Deploy extends AbstractCommandExecutor {

	public static final String DEPLOY_LOCAL_INSTRUCTION = "deployLocal";
	
	public Deploy(EntaxyProducerService entaxyProducerService) {
		super(entaxyProducerService);
	}

	@Override
	protected boolean doExecute(ProducerResult currentResult, CommandResult commandResult,
			Map<String, Object> instructions) throws Exception {
		
		boolean deployLocal = false;
		
		Object deployLocalValue = instructions.get(DEPLOY_LOCAL_INSTRUCTION);
		if (deployLocalValue != null)
			if (deployLocalValue instanceof Boolean)
				deployLocal = (Boolean)deployLocalValue;
		
		ArtifactList artifactList = currentResult.findResultObject(ArtifactList.class);
		DeployedArtifactList deployedArtifactList = new DeployedArtifactList();
		
		ArtifactService artifactService = ArtifactsHelper.getInstance().getArtifactService();
		
		for (Artifact artifact: artifactList) {
			DeployedArtifact da = deployLocal
					?artifactService.deployLocal(artifact)
					:artifactService.deployShared(artifact);
			deployedArtifactList.add(da);
		}

		printOutput("\n\n\t== DEPLOY ==\n");
		printOutput("DEPLOYED: [" 
				+ deployedArtifactList.stream().map(da->da.getLocation()).collect(Collectors.joining(","))
				+ "]");
		
		commandResult.resultObject(deployedArtifactList);
		
		return true;
	}

}
