/*-
 * ~~~~~~licensing~~~~~~
 * test-producers
 * ==========
 * Copyright (C) 2020 - 2022 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.commands;

import java.util.Map;

import ru.entaxy.esb.platform.runtime.base.connecting.generator.Generated;
import ru.entaxy.platform.core.producer.api.EntaxyProducerService;
import ru.entaxy.platform.core.producer.api.ProducerResult;
import ru.entaxy.platform.core.producer.api.ProducerResult.CommandResult;
import ru.entaxy.platform.core.producer.executor.AbstractCommandExecutor;
import ru.entaxy.platform.core.producer.executor.CommandExecutor;
import ru.entaxy.platform.core.producer.executor.generationmodel.AbstractTask;
import ru.entaxy.platform.core.producer.executor.generationmodel.GeneratedList;
import ru.entaxy.platform.core.producer.executor.generationmodel.GenerationModel;

@CommandExecutor(id = "generate", predecessors = {"validate", "pre-generate"})
public class Generate extends AbstractCommandExecutor {

	public Generate(EntaxyProducerService entaxyProducerService) {
		super(entaxyProducerService);
		// TODO Auto-generated constructor stub
	}

	@Override
	protected boolean doExecute(ProducerResult currentResult, CommandResult commandResult,
			Map<String, Object> instructions) throws Exception {
		
		GenerationModel generationModel = currentResult.findResultObject(GenerationModel.class);

		// generate
		this.printOutput("\n\t == Execution ==\n");
		for (AbstractTask at: generationModel.getTasks()) {
			at.setPrintOutput(isPrintOutput);
			at.execute(currentResult, commandResult, entaxyProducerService, instructions);
			this.printOutput("\n :: Result ::\n");
			if (at.getGeneratedResult()==null)
				this.printOutput("-> NOTHING");
			else {
				Generated g = at.getGeneratedResult();
				this.printOutput("\tTYPE :: [" + g.getType() + "]");
				this.printOutput("\tCONTENT :: \n\n");
				if (g.getObject() == null)
					this.printOutput("-> EMPTY\n\n");
				else
					this.printOutput(g.getObject().toString());
			}
			this.printOutput("\n ::\n");
		}
		this.printOutput("\n\t ==\n");
		
		// collect result
		GeneratedList resultObject = new GeneratedList();
		for (AbstractTask at: generationModel.getTasks())
			resultObject.add(at.getGeneratedResult());
		commandResult.resultObject(resultObject);	
			
		return true;
	}

}
