/*-
 * ~~~~~~licensing~~~~~~
 * object-producer-core
 * ==========
 * Copyright (C) 2020 - 2022 EmDev LLC
 * ==========
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.executor.generationmodel;

import java.util.HashMap;
import java.util.Map;

import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import ru.entaxy.platform.base.objects.EntaxyObject;
import ru.entaxy.platform.base.objects.EntaxyObjectService;

@Component(service = ObjectResolveHelper.class, immediate = true)
public class ObjectResolveHelper {

	public static ObjectResolveHelper _INSTANCE;
	
	public static ObjectResolveHelper getInstance(){
		return _INSTANCE;
	}
	
	@Reference(cardinality = ReferenceCardinality.MANDATORY)
	EntaxyObjectService entaxyObjectService;
	
	protected Map<String, ObjectDataResolver> externalResolvers = new HashMap<>();
	
	DefaultObjectDataResolver defaultObjectDataResolver = new DefaultObjectDataResolver();
	
	@Activate
	public void activate() {
		_INSTANCE = this;
	}
	
	@Reference(unbind = "removeResolver", cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC,
			policyOption = ReferencePolicyOption.GREEDY, collectionType = CollectionType.SERVICEOBJECTS, service = ObjectDataResolver.class)
	public void addResolver(ObjectDataResolver resolver) {
		synchronized (externalResolvers) {
			externalResolvers.put(resolver.getSupportedObjectType(), resolver);
		}
	}

	public void removeResolver(ObjectDataResolver resolver) {
		synchronized (externalResolvers) {
			externalResolvers.remove(resolver.getSupportedObjectType());
		}
	}
	
	public EntaxyObject findObject(String objectId, String objectType) {
		return entaxyObjectService.getObjects().stream()
				.filter(obj -> (obj.getObjectId().equals(objectId) && obj.getObjectType().equals(objectType)))
				.findFirst().orElse(null);
	}
	
	public ObjectDataResolver getResolver(EntaxyObject object) {
		return externalResolvers.getOrDefault(object.getObjectType(), defaultObjectDataResolver);
	}
	
	public Object resolve(EntaxyObject entaxyObject, String propertyName) {
		return getResolver(entaxyObject).resolve(entaxyObject, propertyName);
	}
}
